import sys
from PyQt5.QtWidgets import (
    QApplication, QMainWindow, QWidget, QVBoxLayout, QHBoxLayout,
    QPushButton, QLabel, QFrame, QSizePolicy, QStyle, QTextEdit, QSplitter
)
from PyQt5.QtCore import Qt, QSize, QTimer, pyqtSignal
from PyQt5.QtGui import QFont, QTextCursor
import time
import threading
from phone_manager import PhoneManager
from waveform_widget import WaveformWidget
from phone_state import PhoneState

class PhoneUI(QMainWindow):
    debug_signal = pyqtSignal(str)
    
    def __init__(self):
        super().__init__()
        self.setWindowTitle("DryBox - Noise XK + Codec2 + 4FSK")
        self.setGeometry(100, 100, 1200, 900)
        
        # Set minimum size to ensure window is resizable
        self.setMinimumSize(800, 600)
        
        # Auto test state
        self.auto_test_running = False
        self.auto_test_timer = None
        self.test_step = 0
        self.setStyleSheet("""
            QMainWindow { background-color: #333333; }
            QLabel { color: #E0E0E0; font-size: 14px; }
            QPushButton {
                background-color: #0078D4; color: white; border: none;
                padding: 10px 15px; border-radius: 5px; font-size: 14px;
                min-height: 30px;
            }
            QPushButton:hover { background-color: #005A9E; }
            QPushButton:pressed { background-color: #003C6B; }
            QPushButton#settingsButton { background-color: #555555; }
            QPushButton#settingsButton:hover { background-color: #777777; }
            QFrame#phoneDisplay {
                background-color: #1E1E1E; border: 2px solid #0078D4;
                border-radius: 10px;
            }
            QLabel#phoneTitleLabel {
                font-size: 18px; font-weight: bold; padding-bottom: 5px;
                color: #FFFFFF;
            }
            QLabel#mainTitleLabel {
                font-size: 24px; font-weight: bold; color: #00A2E8;
                padding: 15px;
            }
            QWidget#phoneWidget {
                border: 1px solid #4A4A4A; border-radius: 8px;
                padding: 10px; background-color: #3A3A3A;
            }
            QTextEdit#debugConsole {
                background-color: #1E1E1E; color: #00FF00; 
                font-family: monospace; font-size: 12px;
                border: 2px solid #0078D4; border-radius: 5px;
            }
            QPushButton#autoTestButton {
                background-color: #FF8C00; min-height: 35px;
            }
            QPushButton#autoTestButton:hover { background-color: #FF7F00; }
        """)

        # Setup debug signal early
        self.debug_signal.connect(self.append_debug)
        
        self.manager = PhoneManager()
        self.manager.ui = self  # Set UI reference for debug logging
        self.manager.initialize_phones()

        # Main widget with splitter
        main_widget = QWidget()
        self.setCentralWidget(main_widget)
        main_layout = QVBoxLayout()
        main_widget.setLayout(main_layout)
        
        # Create splitter for phones and debug console
        self.splitter = QSplitter(Qt.Vertical)
        main_layout.addWidget(self.splitter)
        
        # Top widget for phones
        phones_widget = QWidget()
        phones_layout = QVBoxLayout()
        phones_layout.setSpacing(20)
        phones_layout.setContentsMargins(20, 20, 20, 20)
        phones_layout.setAlignment(Qt.AlignCenter)
        phones_widget.setLayout(phones_layout)

        # App Title
        app_title_label = QLabel("Integrated Protocol Control Panel")
        app_title_label.setObjectName("mainTitleLabel")
        app_title_label.setAlignment(Qt.AlignCenter)
        phones_layout.addWidget(app_title_label)
        
        # Protocol info
        protocol_info = QLabel("Noise XK + Codec2 (1200bps) + 4FSK")
        protocol_info.setAlignment(Qt.AlignCenter)
        protocol_info.setStyleSheet("font-size: 12px; color: #00A2E8;")
        phones_layout.addWidget(protocol_info)

        # Phone displays layout
        phone_controls_layout = QHBoxLayout()
        phone_controls_layout.setSpacing(30)
        phone_controls_layout.setAlignment(Qt.AlignCenter)
        phones_layout.addLayout(phone_controls_layout)

        # Setup UI for phones
        for phone in self.manager.phones:
            phone_container_widget, phone_display_frame, phone_button, waveform_widget, sent_waveform_widget, phone_status_label = self._create_phone_ui(
                f"Phone {phone['id']+1}", lambda checked, pid=phone['id']: self.manager.phone_action(pid, self)
            )
            phone['button'] = phone_button
            phone['waveform'] = waveform_widget
            phone['sent_waveform'] = sent_waveform_widget
            phone['status_label'] = phone_status_label
            phone_controls_layout.addWidget(phone_container_widget)
            phone['client'].data_received.connect(lambda data, cid=phone['id']: self.manager.update_waveform(cid, data))
            phone['client'].state_changed.connect(lambda state, num, cid=phone['id']: self.set_phone_state(cid, state, num))
            phone['client'].start()

        # Control buttons layout
        control_layout = QHBoxLayout()
        control_layout.setSpacing(20)
        
        # Auto Test Button
        self.auto_test_button = QPushButton("🧪 Run Automatic Test")
        self.auto_test_button.setObjectName("autoTestButton")
        self.auto_test_button.setFixedWidth(200)
        self.auto_test_button.clicked.connect(self.toggle_auto_test)
        control_layout.addWidget(self.auto_test_button)
        
        # Clear Debug Button
        self.clear_debug_button = QPushButton("Clear Debug")
        self.clear_debug_button.setFixedWidth(120)
        self.clear_debug_button.clicked.connect(self.clear_debug)
        control_layout.addWidget(self.clear_debug_button)
        
        # Settings Button
        self.settings_button = QPushButton("Settings")
        self.settings_button.setObjectName("settingsButton")
        self.settings_button.setFixedWidth(120)
        self.settings_button.setIcon(self.style().standardIcon(QStyle.SP_FileDialogDetailedView))
        self.settings_button.setIconSize(QSize(20, 20))
        self.settings_button.clicked.connect(self.settings_action)
        control_layout.addWidget(self.settings_button)
        
        phones_layout.addLayout(control_layout)
        
        # Add phones widget to splitter
        self.splitter.addWidget(phones_widget)
        
        # Debug console
        self.debug_console = QTextEdit()
        self.debug_console.setObjectName("debugConsole")
        self.debug_console.setReadOnly(True)
        self.debug_console.setMinimumHeight(200)
        self.debug_console.setMaximumHeight(400)
        self.splitter.addWidget(self.debug_console)
        
        # Flush any queued debug messages
        if hasattr(self, '_debug_queue'):
            for msg in self._debug_queue:
                self.debug_console.append(msg)
            del self._debug_queue
        
        # Set splitter sizes (70% phones, 30% debug)
        self.splitter.setSizes([600, 300])

        # Initialize UI
        for phone in self.manager.phones:
            self.update_phone_ui(phone['id'])
            
        # Initial debug message
        QTimer.singleShot(100, lambda: self.debug("DryBox UI initialized with integrated protocol"))

    def _create_phone_ui(self, title, action_slot):
        phone_container_widget = QWidget()
        phone_container_widget.setObjectName("phoneWidget")
        phone_layout = QVBoxLayout()
        phone_layout.setAlignment(Qt.AlignCenter)
        phone_layout.setSpacing(15)
        phone_container_widget.setLayout(phone_layout)

        phone_title_label = QLabel(title)
        phone_title_label.setObjectName("phoneTitleLabel")
        phone_title_label.setAlignment(Qt.AlignCenter)
        phone_layout.addWidget(phone_title_label)

        phone_display_frame = QFrame()
        phone_display_frame.setObjectName("phoneDisplay")
        phone_display_frame.setFixedSize(220, 300)
        phone_display_frame.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)

        display_content_layout = QVBoxLayout(phone_display_frame)
        display_content_layout.setAlignment(Qt.AlignCenter)
        phone_status_label = QLabel("Idle")
        phone_status_label.setAlignment(Qt.AlignCenter)
        phone_status_label.setFont(QFont("Arial", 16))
        display_content_layout.addWidget(phone_status_label)
        phone_layout.addWidget(phone_display_frame, alignment=Qt.AlignCenter)

        phone_button = QPushButton()
        phone_button.setFixedWidth(120)
        phone_button.setIconSize(QSize(20, 20))
        phone_button.clicked.connect(action_slot)
        phone_layout.addWidget(phone_button, alignment=Qt.AlignCenter)

        # Received waveform
        waveform_label = QLabel(f"{title} Received")
        waveform_label.setAlignment(Qt.AlignCenter)
        waveform_label.setStyleSheet("font-size: 12px; color: #E0E0E0;")
        phone_layout.addWidget(waveform_label)
        waveform_widget = WaveformWidget(dynamic=False)
        waveform_widget.setFixedSize(220, 60)
        phone_layout.addWidget(waveform_widget, alignment=Qt.AlignCenter)

        # Sent waveform
        sent_waveform_label = QLabel(f"{title} Sent")
        sent_waveform_label.setAlignment(Qt.AlignCenter)
        sent_waveform_label.setStyleSheet("font-size: 12px; color: #E0E0E0;")
        phone_layout.addWidget(sent_waveform_label)
        sent_waveform_widget = WaveformWidget(dynamic=False)
        sent_waveform_widget.setFixedSize(220, 60)
        phone_layout.addWidget(sent_waveform_widget, alignment=Qt.AlignCenter)

        return phone_container_widget, phone_display_frame, phone_button, waveform_widget, sent_waveform_widget, phone_status_label

    def update_phone_ui(self, phone_id):
        phone = self.manager.phones[phone_id]
        other_phone = self.manager.phones[1 - phone_id]
        state = phone['state']
        phone_number = other_phone['number'] if state != PhoneState.IDLE else ""
        button = phone['button']
        status_label = phone['status_label']

        if state == PhoneState.IDLE:
            button.setText("Call")
            button.setIcon(self.style().standardIcon(QStyle.SP_MediaPlay))
            status_label.setText("Idle")
            button.setStyleSheet("background-color: #0078D4;")
        elif state == PhoneState.CALLING:
            button.setText("Cancel")
            button.setIcon(self.style().standardIcon(QStyle.SP_MediaStop))
            status_label.setText(f"Calling {phone_number}...")
            button.setStyleSheet("background-color: #E81123;")
        elif state == PhoneState.IN_CALL:
            button.setText("Hang Up")
            button.setIcon(self.style().standardIcon(QStyle.SP_DialogCancelButton))
            status_label.setText(f"In Call with {phone_number}")
            button.setStyleSheet("background-color: #E81123;")
        elif state == PhoneState.RINGING:
            button.setText("Answer")
            button.setIcon(self.style().standardIcon(QStyle.SP_DialogApplyButton))
            status_label.setText(f"Incoming Call from {phone_number}")
            button.setStyleSheet("background-color: #107C10;")

    def set_phone_state(self, client_id, state_str, number):
        self.debug(f"Phone {client_id + 1} state change: {state_str}")
        
        # Handle protocol-specific states
        if state_str == "HANDSHAKE_COMPLETE":
            phone = self.manager.phones[client_id]
            phone['status_label'].setText("🔒 Secure Channel Established")
            self.debug(f"Phone {client_id + 1} secure channel established")
            self.manager.start_audio(client_id, parent=self)
            return
        elif state_str == "VOICE_START":
            phone = self.manager.phones[client_id]
            phone['status_label'].setText("🎤 Voice Active (Encrypted)")
            self.debug(f"Phone {client_id + 1} voice session started")
            return
        elif state_str == "VOICE_END":
            phone = self.manager.phones[client_id]
            phone['status_label'].setText("🔒 Secure Channel")
            self.debug(f"Phone {client_id + 1} voice session ended")
            return
            
        # Handle regular states
        state = self.manager.map_state(state_str)
        phone = self.manager.phones[client_id]
        other_phone = self.manager.phones[1 - client_id]
        self.debug(f"Setting state for Phone {client_id + 1}: {state.name if hasattr(state, 'name') else state}, number: {number}, is_initiator: {phone['is_initiator']}")
        phone['state'] = state
        if state == PhoneState.IN_CALL:
            self.debug(f"Phone {client_id + 1} confirmed in IN_CALL state")
            self.debug(f"  state_str={state_str}, number={number}")
            self.debug(f"  is_initiator={phone['is_initiator']}")
            
            # Only start handshake when the initiator RECEIVES the IN_CALL message
            if state_str == "IN_CALL" and phone['is_initiator']:
                self.debug(f"Phone {client_id + 1} (initiator) received IN_CALL, starting handshake")
                phone['client'].start_handshake(initiator=True, keypair=phone['keypair'], peer_pubkey=other_phone['public_key'])
            elif number == "HANDSHAKE":
                # Old text-based handshake trigger - no longer used
                self.debug(f"Phone {client_id + 1} received legacy HANDSHAKE message")
            elif number == "HANDSHAKE_DONE":
                self.debug(f"Phone {client_id + 1} received HANDSHAKE_DONE")
                # Handled by HANDSHAKE_COMPLETE now
                pass
        self.update_phone_ui(client_id)

    def settings_action(self):
        print("Settings clicked")
        self.debug("Settings clicked")

    def debug(self, message):
        """Thread-safe debug logging to both console and UI"""
        timestamp = time.strftime("%H:%M:%S.%f")[:-3]
        debug_msg = f"[{timestamp}] {message}"
        print(debug_msg)  # Console output
        self.debug_signal.emit(debug_msg)  # UI output
    
    def append_debug(self, message):
        """Append debug message to console (called from main thread)"""
        if hasattr(self, 'debug_console'):
            self.debug_console.append(message)
            # Auto-scroll to bottom
            cursor = self.debug_console.textCursor()
            cursor.movePosition(QTextCursor.End)
            self.debug_console.setTextCursor(cursor)
        else:
            # Queue messages until console is ready
            if not hasattr(self, '_debug_queue'):
                self._debug_queue = []
            self._debug_queue.append(message)
    
    def clear_debug(self):
        """Clear debug console"""
        self.debug_console.clear()
        self.debug("Debug console cleared")
    
    def toggle_auto_test(self):
        """Toggle automatic test sequence"""
        if not self.auto_test_running:
            self.start_auto_test()
        else:
            self.stop_auto_test()
    
    def start_auto_test(self):
        """Start automatic test sequence"""
        self.auto_test_running = True
        self.auto_test_button.setText("⏹ Stop Test")
        self.test_step = 0
        
        self.debug("=== STARTING AUTOMATIC TEST SEQUENCE ===")
        self.debug("Test will go through complete protocol flow")
        
        # Start test timer
        self.auto_test_timer = QTimer()
        self.auto_test_timer.timeout.connect(self.execute_test_step)
        self.auto_test_timer.start(2000)  # 2 second intervals
        
        # Execute first step immediately
        self.execute_test_step()
    
    def stop_auto_test(self):
        """Stop automatic test sequence"""
        self.auto_test_running = False
        self.auto_test_button.setText("🧪 Run Automatic Test")
        
        if self.auto_test_timer:
            self.auto_test_timer.stop()
            self.auto_test_timer = None
        
        self.debug("=== TEST SEQUENCE STOPPED ===")
    
    def execute_test_step(self):
        """Execute next step in test sequence"""
        phone1 = self.manager.phones[0]
        phone2 = self.manager.phones[1]
        
        self.debug(f"\n--- Test Step {self.test_step + 1} ---")
        
        if self.test_step == 0:
            # Step 1: Check initial state
            self.debug("Checking initial state...")
            state1 = phone1['state']
            state2 = phone2['state']
            # Handle both enum and int states
            state1_name = state1.name if hasattr(state1, 'name') else str(state1)
            state2_name = state2.name if hasattr(state2, 'name') else str(state2)
            self.debug(f"Phone 1 state: {state1_name}")
            self.debug(f"Phone 2 state: {state2_name}")
            self.debug(f"Phone 1 connected: {phone1['client'].sock is not None}")
            self.debug(f"Phone 2 connected: {phone2['client'].sock is not None}")
            
        elif self.test_step == 1:
            # Step 2: Make call
            self.debug("Phone 1 calling Phone 2...")
            self.manager.phone_action(0, self)
            state1_name = phone1['state'].name if hasattr(phone1['state'], 'name') else str(phone1['state'])
            state2_name = phone2['state'].name if hasattr(phone2['state'], 'name') else str(phone2['state'])
            self.debug(f"Phone 1 state after call: {state1_name}")
            self.debug(f"Phone 2 state after call: {state2_name}")
            
        elif self.test_step == 2:
            # Step 3: Answer call
            self.debug("Phone 2 answering call...")
            self.manager.phone_action(1, self)
            state1_name = phone1['state'].name if hasattr(phone1['state'], 'name') else str(phone1['state'])
            state2_name = phone2['state'].name if hasattr(phone2['state'], 'name') else str(phone2['state'])
            self.debug(f"Phone 1 state after answer: {state1_name}")
            self.debug(f"Phone 2 state after answer: {state2_name}")
            self.debug(f"Phone 1 is_initiator: {phone1['is_initiator']}")
            self.debug(f"Phone 2 is_initiator: {phone2['is_initiator']}")
            
        elif self.test_step == 3:
            # Step 4: Check handshake progress
            self.debug("Checking handshake progress...")
            self.debug(f"Phone 1 handshake in progress: {phone1['client'].state.handshake_in_progress}")
            self.debug(f"Phone 2 handshake in progress: {phone2['client'].state.handshake_in_progress}")
            self.debug(f"Phone 1 command queue: {phone1['client'].state.command_queue.qsize()}")
            self.debug(f"Phone 2 command queue: {phone2['client'].state.command_queue.qsize()}")
            # Increase timer interval for handshake
            self.auto_test_timer.setInterval(3000)  # 3 seconds
            
        elif self.test_step == 4:
            # Step 5: Check handshake status
            self.debug("Checking Noise XK handshake status...")
            self.debug(f"Phone 1 handshake complete: {phone1['client'].handshake_complete}")
            self.debug(f"Phone 2 handshake complete: {phone2['client'].handshake_complete}")
            self.debug(f"Phone 1 has session: {phone1['client'].noise_session is not None}")
            self.debug(f"Phone 2 has session: {phone2['client'].noise_session is not None}")
            # Reset timer interval
            self.auto_test_timer.setInterval(2000)
            
        elif self.test_step == 5:
            # Step 6: Check voice status
            self.debug("Checking voice session status...")
            self.debug(f"Phone 1 voice active: {phone1['client'].voice_active}")
            self.debug(f"Phone 2 voice active: {phone2['client'].voice_active}")
            self.debug(f"Phone 1 codec initialized: {phone1['client'].codec is not None}")
            self.debug(f"Phone 2 codec initialized: {phone2['client'].codec is not None}")
            self.debug(f"Phone 1 modem initialized: {phone1['client'].modem is not None}")
            self.debug(f"Phone 2 modem initialized: {phone2['client'].modem is not None}")
            
        elif self.test_step == 6:
            # Step 7: Check audio transmission
            self.debug("Checking audio transmission...")
            self.debug(f"Phone 1 audio file loaded: {phone1['audio_file'] is not None}")
            self.debug(f"Phone 2 audio file loaded: {phone2['audio_file'] is not None}")
            self.debug(f"Phone 1 frame counter: {phone1.get('frame_counter', 0)}")
            self.debug(f"Phone 2 frame counter: {phone2.get('frame_counter', 0)}")
            self.debug(f"Phone 1 audio timer active: {phone1['audio_timer'] is not None and phone1['audio_timer'].isActive()}")
            self.debug(f"Phone 2 audio timer active: {phone2['audio_timer'] is not None and phone2['audio_timer'].isActive()}")
            
        elif self.test_step == 7:
            # Step 8: Protocol details
            self.debug("Protocol stack details:")
            if phone1['client'].codec:
                self.debug(f"Codec mode: {phone1['client'].codec.mode.name}")
                self.debug(f"Frame size: {phone1['client'].codec.frame_bits} bits")
                self.debug(f"Frame duration: {phone1['client'].codec.frame_ms} ms")
            if phone1['client'].modem:
                self.debug(f"FSK frequencies: {phone1['client'].modem.frequencies}")
                self.debug(f"Symbol rate: {phone1['client'].modem.baud_rate} baud")
            
        elif self.test_step == 8:
            # Step 9: Wait for more frames
            self.debug("Letting voice transmission run...")
            self.auto_test_timer.setInterval(5000)  # Wait 5 seconds
            
        elif self.test_step == 9:
            # Step 10: Final statistics
            self.debug("Final transmission statistics:")
            self.debug(f"Phone 1 frames sent: {phone1.get('frame_counter', 0)}")
            self.debug(f"Phone 2 frames sent: {phone2.get('frame_counter', 0)}")
            self.auto_test_timer.setInterval(2000)  # Back to 2 seconds
            
        elif self.test_step == 10:
            # Step 11: Hang up
            self.debug("Hanging up call...")
            self.manager.phone_action(0, self)
            state1_name = phone1['state'].name if hasattr(phone1['state'], 'name') else str(phone1['state'])
            state2_name = phone2['state'].name if hasattr(phone2['state'], 'name') else str(phone2['state'])
            self.debug(f"Phone 1 state after hangup: {state1_name}")
            self.debug(f"Phone 2 state after hangup: {state2_name}")
            
        elif self.test_step == 11:
            # Complete
            self.debug("\n=== TEST SEQUENCE COMPLETE ===")
            self.debug("All protocol components tested successfully!")
            self.stop_auto_test()
            return
        
        self.test_step += 1

    def closeEvent(self, event):
        if self.auto_test_running:
            self.stop_auto_test()
        for phone in self.manager.phones:
            phone['client'].stop()
        event.accept()

if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = PhoneUI()
    window.show()
    sys.exit(app.exec_())