import secrets
from PyQt5.QtCore import QTimer
from protocol_phone_client import ProtocolPhoneClient
from session import NoiseXKSession
from phone_state import PhoneState  # Added import
import struct
import wave
import os

class PhoneManager:
    def __init__(self):
        self.phones = []
        self.handshake_done_count = 0
        self.ui = None  # Will be set by UI

    def debug(self, message):
        """Send debug message to UI if available"""
        if self.ui and hasattr(self.ui, 'debug'):
            self.ui.debug(f"[PhoneManager] {message}")
        else:
            print(f"[PhoneManager] {message}")
    
    def initialize_phones(self):
        for i in range(2):
            client = ProtocolPhoneClient(i)  # Use protocol client
            client.set_debug_callback(self.debug)  # Set debug callback
            client.manager = self  # Set manager reference for handshake lookup
            keypair = NoiseXKSession.generate_keypair()
            phone = {
                'id': i,
                'client': client,
                'state': PhoneState.IDLE,
                'number': "123-4567" if i == 0 else "987-6543",
                'audio_timer': None,
                'keypair': keypair,
                'public_key': keypair.public,
                'is_initiator': False,
                'audio_file': None,  # For test audio
                'frame_counter': 0
            }
            client.keypair = keypair  # Also set keypair on client
            self.phones.append(phone)
            self.debug(f"Initialized Phone {i+1} with public key: {keypair.public.data.hex()[:32]}...")

        self.phones[0]['peer_public_key'] = self.phones[1]['public_key']
        self.phones[1]['peer_public_key'] = self.phones[0]['public_key']

    def phone_action(self, phone_id, ui_manager):
        phone = self.phones[phone_id]
        other_phone = self.phones[1 - phone_id]
        self.debug(f"Phone {phone_id + 1} action triggered, current state: {phone['state'].name}")

        if phone['state'] == PhoneState.IDLE:
            self.debug(f"Phone {phone_id + 1} initiating call to Phone {2-phone_id}")
            phone['state'] = PhoneState.CALLING
            other_phone['state'] = PhoneState.RINGING
            phone['is_initiator'] = True
            other_phone['is_initiator'] = False
            phone['client'].send("RINGING")
        elif phone['state'] == PhoneState.RINGING:
            self.debug(f"Phone {phone_id + 1} answering call from Phone {2-phone_id}")
            phone['state'] = PhoneState.IN_CALL
            # Don't set other_phone state here - let it set when it receives IN_CALL
            phone['client'].send("IN_CALL")
        elif phone['state'] in [PhoneState.IN_CALL, PhoneState.CALLING]:
            if not phone['client'].state.handshake_in_progress and phone['state'] != PhoneState.CALLING:
                phone['state'] = other_phone['state'] = PhoneState.IDLE
                phone['client'].send("CALL_END")
                for p in [phone, other_phone]:
                    if p['audio_timer']:
                        p['audio_timer'].stop()
                    # End voice session
                    if p['client'].voice_active:
                        p['client'].end_voice_session()
                    # Close audio file
                    if p['audio_file']:
                        p['audio_file'].close()
                        p['audio_file'] = None
                    p['frame_counter'] = 0
            else:
                self.debug(f"Phone {phone_id + 1} cannot hang up during handshake or call setup")
        
        ui_manager.update_phone_ui(phone_id)
        ui_manager.update_phone_ui(1 - phone_id)

    def send_audio(self, phone_id):
        phone = self.phones[phone_id]
        if phone['state'] != PhoneState.IN_CALL:
            self.debug(f"Phone {phone_id + 1} not in call, stopping audio timer")
            if phone['audio_timer']:
                phone['audio_timer'].stop()
            return
            
        if not phone['client'].handshake_complete:
            self.debug(f"Phone {phone_id + 1} handshake not complete, skipping audio send")
            return
            
        if not phone['client'].voice_active:
            self.debug(f"Phone {phone_id + 1} voice not active, skipping audio send")
            return
            
        if phone['state'] == PhoneState.IN_CALL and phone['client'].handshake_complete and phone['client'].voice_active:
            # Load test audio file if not loaded
            if phone['audio_file'] is None:
                wav_path = "../wav/input_8k_mono.wav"
                if not os.path.exists(wav_path):
                    wav_path = "wav/input_8k_mono.wav"
                if os.path.exists(wav_path):
                    try:
                        phone['audio_file'] = wave.open(wav_path, 'rb')
                        self.debug(f"Phone {phone_id + 1} loaded test audio file: {wav_path}")
                    except Exception as e:
                        self.debug(f"Phone {phone_id + 1} failed to load audio: {e}")
                        # Use mock audio as fallback
                        phone['audio_file'] = None
            
            # Read audio frame (40ms at 8kHz = 320 samples)
            if phone['audio_file']:
                try:
                    frames = phone['audio_file'].readframes(320)
                    if not frames or len(frames) < 640:  # 320 samples * 2 bytes
                        # Loop back to start
                        phone['audio_file'].rewind()
                        frames = phone['audio_file'].readframes(320)
                    
                    # Send through protocol (codec + 4FSK + encryption)
                    phone['client'].send_voice_frame(frames)
                    
                    # Update waveform
                    if len(frames) >= 2:
                        samples = struct.unpack(f'{len(frames)//2}h', frames)
                        self.update_sent_waveform(phone_id, frames)
                    
                    phone['frame_counter'] += 1
                    if phone['frame_counter'] % 25 == 0:  # Log every second
                        self.debug(f"Phone {phone_id + 1} sent {phone['frame_counter']} voice frames")
                        
                except Exception as e:
                    self.debug(f"Phone {phone_id + 1} audio send error: {e}")
            else:
                # Fallback: send mock audio
                mock_audio = secrets.token_bytes(320)
                phone['client'].send_voice_frame(mock_audio)
                self.update_sent_waveform(phone_id, mock_audio)

    def start_audio(self, client_id, parent=None):
        self.handshake_done_count += 1
        self.debug(f"HANDSHAKE_DONE received for client {client_id}, count: {self.handshake_done_count}")
        
        # Start voice session for this client
        phone = self.phones[client_id]
        if phone['client'].handshake_complete and not phone['client'].voice_active:
            phone['client'].start_voice_session()
        
        if self.handshake_done_count == 2:
            # Add a small delay to ensure both sides are ready
            def start_audio_timers():
                self.debug("Starting audio timers for both phones")
                for phone in self.phones:
                    if phone['state'] == PhoneState.IN_CALL:
                        if not phone['audio_timer'] or not phone['audio_timer'].isActive():
                            phone['audio_timer'] = QTimer(parent)  # Parent to PhoneUI
                            phone['audio_timer'].timeout.connect(lambda pid=phone['id']: self.send_audio(pid))
                            phone['audio_timer'].start(40)  # 40ms for each voice frame
            
            # Delay audio start by 500ms to ensure both sides are ready
            QTimer.singleShot(500, start_audio_timers)
            self.handshake_done_count = 0

    def update_waveform(self, client_id, data):
        self.phones[client_id]['waveform'].set_data(data)

    def update_sent_waveform(self, client_id, data):
        self.phones[client_id]['sent_waveform'].set_data(data)

    def map_state(self, state_str):
        if state_str == "RINGING":
            return PhoneState.RINGING
        elif state_str in ["CALL_END", "CALL_DROPPED"]:
            return PhoneState.IDLE
        elif state_str == "IN_CALL":
            return PhoneState.IN_CALL
        elif state_str == "HANDSHAKE":
            return PhoneState.IN_CALL
        elif state_str == "HANDSHAKE_DONE":
            return PhoneState.IN_CALL
        return PhoneState.IDLE