#!/usr/bin/env python3
"""Complete integration test of DryBox with Noise XK, Codec2, and 4FSK"""

import sys
import time
sys.path.append('UI')
from PyQt5.QtWidgets import QApplication
from PyQt5.QtCore import QTimer
from main import PhoneUI

def main():
    app = QApplication(sys.argv)
    window = PhoneUI()
    
    # Track comprehensive results
    results = {
        'handshakes': 0,
        'voice_sessions': 0,
        'frames_sent_p1': 0,
        'frames_sent_p2': 0,
        'frames_received_p1': 0,
        'frames_received_p2': 0,
        'decode_errors': 0,
        'low_confidence': 0
    }
    
    original_debug = window.debug
    def track_debug(msg):
        if "handshake complete!" in msg:
            results['handshakes'] += 1
            original_debug(msg)
        elif "Voice session started" in msg:
            results['voice_sessions'] += 1
            original_debug(msg)
        elif "Encoding voice frame #" in msg:
            if "[Phone1]" in msg:
                results['frames_sent_p1'] += 1
            else:
                results['frames_sent_p2'] += 1
            if "#0" in msg or "#50" in msg or "#100" in msg:
                original_debug(msg)
        elif "Received voice data frame #" in msg:
            if "[Phone1]" in msg:
                results['frames_received_p1'] += 1
            else:
                results['frames_received_p2'] += 1
            if "#0" in msg or "#25" in msg:
                original_debug(msg)
        elif "Voice decode error:" in msg:
            results['decode_errors'] += 1
        elif "Low confidence demodulation:" in msg:
            results['low_confidence'] += 1
    
    window.debug = track_debug
    window.show()
    
    print("\n=== COMPLETE PROTOCOL INTEGRATION TEST ===")
    print("Components:")
    print("- Noise XK handshake and encryption")
    print("- Codec2 voice codec (1200 bps)")
    print("- 4FSK modulation (600/1200/1800/2400 Hz)")
    print("- Message framing for GSM transport")
    print("- Bidirectional voice communication\n")
    
    # Test sequence
    def start_test():
        print("Step 1: Initiating call from Phone 1 to Phone 2...")
        window.manager.phone_action(0, window)
        QTimer.singleShot(1500, answer_call)
    
    def answer_call():
        print("Step 2: Phone 2 answering call...")
        window.manager.phone_action(1, window)
        print("Step 3: Establishing secure channel and starting voice...")
        QTimer.singleShot(10000, show_results)
    
    def show_results():
        phone1 = window.manager.phones[0]
        phone2 = window.manager.phones[1]
        
        print(f"\n=== FINAL RESULTS ===")
        print(f"\nHandshake Status:")
        print(f"  Handshakes completed: {results['handshakes']}")
        print(f"  Phone 1: {'✓' if phone1['client'].handshake_complete else '✗'}")
        print(f"  Phone 2: {'✓' if phone2['client'].handshake_complete else '✗'}")
        
        print(f"\nVoice Session Status:")
        print(f"  Sessions started: {results['voice_sessions']}")
        print(f"  Phone 1 active: {'✓' if phone1['client'].voice_active else '✗'}")
        print(f"  Phone 2 active: {'✓' if phone2['client'].voice_active else '✗'}")
        
        print(f"\nVoice Frame Statistics:")
        print(f"  Phone 1: Sent {results['frames_sent_p1']}, Received {phone1['client'].voice_frame_counter}")
        print(f"  Phone 2: Sent {results['frames_sent_p2']}, Received {phone2['client'].voice_frame_counter}")
        print(f"  Decode errors: {results['decode_errors']}")
        print(f"  Low confidence frames: {results['low_confidence']}")
        
        # Calculate success
        handshake_ok = results['handshakes'] >= 2
        voice_ok = results['voice_sessions'] >= 2
        p1_rx = phone1['client'].voice_frame_counter > 0
        p2_rx = phone2['client'].voice_frame_counter > 0
        
        print(f"\n=== PROTOCOL STACK STATUS ===")
        print(f"  Noise XK Handshake: {'✓ WORKING' if handshake_ok else '✗ FAILED'}")
        print(f"  Voice Sessions: {'✓ WORKING' if voice_ok else '✗ FAILED'}")
        print(f"  Codec2 + 4FSK (P1→P2): {'✓ WORKING' if p2_rx else '✗ FAILED'}")
        print(f"  Codec2 + 4FSK (P2→P1): {'✓ WORKING' if p1_rx else '✗ FAILED'}")
        
        if handshake_ok and voice_ok and (p1_rx or p2_rx):
            print(f"\n✅ INTEGRATION SUCCESSFUL!")
            print(f"   The protocol stack is working with:")
            print(f"   - Secure Noise XK encrypted channel established")
            print(f"   - Voice codec and modulation operational")
            if p1_rx and p2_rx:
                print(f"   - Full duplex communication achieved")
            else:
                print(f"   - Half duplex communication achieved")
                if not p1_rx:
                    print(f"   - Note: Phone 1 not receiving (may need timing adjustments)")
                if not p2_rx:
                    print(f"   - Note: Phone 2 not receiving (may need timing adjustments)")
        else:
            print(f"\n❌ INTEGRATION FAILED")
            if not handshake_ok:
                print(f"   - Noise XK handshake did not complete")
            if not voice_ok:
                print(f"   - Voice sessions did not start")
            if not p1_rx and not p2_rx:
                print(f"   - No voice frames received by either phone")
        
        app.quit()
    
    QTimer.singleShot(1000, start_test)
    sys.exit(app.exec_())

if __name__ == "__main__":
    main()