#!/usr/bin/env python3
"""Final test of complete protocol integration"""

import sys
import time
sys.path.append('UI')
from PyQt5.QtWidgets import QApplication
from PyQt5.QtCore import QTimer
from main import PhoneUI

def main():
    app = QApplication(sys.argv)
    window = PhoneUI()
    
    # Comprehensive tracking
    stats = {
        'handshakes': 0,
        'voice_sessions': 0,
        'frames_sent': 0,
        'frames_received': 0,
        'decode_errors': 0,
        'demod_success': 0,
        'demod_low_conf': 0
    }
    
    original_debug = window.debug
    def track_events(msg):
        if "handshake complete!" in msg:
            stats['handshakes'] += 1
            original_debug(msg)
        elif "Voice session started" in msg:
            stats['voice_sessions'] += 1
            original_debug(msg)
        elif "Encoding voice frame" in msg:
            stats['frames_sent'] += 1
            if stats['frames_sent'] == 1:
                original_debug("First frame encoded successfully")
        elif "voice frame #" in msg and "Received" in msg:
            stats['frames_received'] += 1
            if stats['frames_received'] == 1:
                original_debug("✓ First voice frame received!")
            elif stats['frames_received'] % 50 == 0:
                original_debug(f"✓ Received {stats['frames_received']} voice frames")
        elif "Voice decode error:" in msg:
            stats['decode_errors'] += 1
            if stats['decode_errors'] == 1:
                original_debug(f"First decode error: {msg}")
        elif "Low confidence demodulation:" in msg:
            stats['demod_low_conf'] += 1
    
    window.debug = track_events
    window.show()
    
    print("\n=== FINAL PROTOCOL INTEGRATION TEST ===")
    print("Testing complete stack:")
    print("- Noise XK handshake")
    print("- Codec2 voice compression")  
    print("- 4FSK modulation")
    print("- Message framing\n")
    
    def start_call():
        print("Step 1: Initiating call...")
        window.manager.phone_action(0, window)
        QTimer.singleShot(1500, answer_call)
    
    def answer_call():
        print("Step 2: Answering call...")
        window.manager.phone_action(1, window)
        print("Step 3: Waiting for voice transmission...\n")
        QTimer.singleShot(8000, final_results)
    
    def final_results():
        phone1 = window.manager.phones[0]
        phone2 = window.manager.phones[1]
        
        print("\n=== FINAL RESULTS ===")
        print(f"\nProtocol Status:")
        print(f"  Handshakes completed: {stats['handshakes']}")
        print(f"  Voice sessions: {stats['voice_sessions']}")
        print(f"  Frames sent: {stats['frames_sent']}")
        print(f"  Frames received: {stats['frames_received']}")
        print(f"  Decode errors: {stats['decode_errors']}")
        print(f"  Low confidence demod: {stats['demod_low_conf']}")
        
        print(f"\nPhone Status:")
        print(f"  Phone 1: handshake={phone1['client'].handshake_complete}, "
              f"voice={phone1['client'].voice_active}, "
              f"rx={phone1['client'].voice_frame_counter}")
        print(f"  Phone 2: handshake={phone2['client'].handshake_complete}, "
              f"voice={phone2['client'].voice_active}, "
              f"rx={phone2['client'].voice_frame_counter}")
        
        # Determine success
        success = (
            stats['handshakes'] >= 2 and
            stats['voice_sessions'] >= 2 and
            stats['frames_received'] > 0 and
            stats['decode_errors'] == 0
        )
        
        if success:
            print(f"\n✅ PROTOCOL INTEGRATION SUCCESSFUL!")
            print(f"   - Noise XK: Working")
            print(f"   - Codec2: Working")
            print(f"   - 4FSK: Working")
            print(f"   - Framing: Working")
            print(f"   - Voice transmission: {stats['frames_received']} frames received")
        else:
            print(f"\n❌ Issues detected:")
            if stats['handshakes'] < 2:
                print(f"   - Handshake incomplete")
            if stats['decode_errors'] > 0:
                print(f"   - Voice decode errors: {stats['decode_errors']}")
            if stats['frames_received'] == 0:
                print(f"   - No voice frames received")
        
        app.quit()
    
    QTimer.singleShot(1000, start_call)
    sys.exit(app.exec_())

if __name__ == "__main__":
    main()