#!/usr/bin/env python3
"""Test with proper message framing"""

import sys
import time
sys.path.append('UI')
from PyQt5.QtWidgets import QApplication
from PyQt5.QtCore import QTimer
from main import PhoneUI

def main():
    app = QApplication(sys.argv)
    window = PhoneUI()
    
    # Track results
    results = {
        'handshakes': 0,
        'voice_started': 0,
        'decrypt_errors': 0,
        'frames_received': 0
    }
    
    original_debug = window.debug
    def count_debug(msg):
        if "handshake complete!" in msg:
            results['handshakes'] += 1
        elif "Voice session started" in msg:
            results['voice_started'] += 1
        elif "Decryption error:" in msg:
            results['decrypt_errors'] += 1
        elif "Received voice data frame" in msg:
            results['frames_received'] += 1
        # Show important messages
        if any(x in msg for x in ["handshake complete!", "Voice session", "frame #0", "frame #25"]):
            original_debug(msg)
    
    window.debug = count_debug
    window.show()
    
    print("\n=== TEST WITH MESSAGE FRAMING ===")
    print("- Proper length-prefixed messages")
    print("- No fragmentation issues\n")
    
    # Test sequence
    def test_sequence():
        print("1. Making call...")
        window.manager.phone_action(0, window)
        QTimer.singleShot(1000, answer_call)
    
    def answer_call():
        print("2. Answering call...")
        window.manager.phone_action(1, window)
        QTimer.singleShot(8000, show_results)
    
    def show_results():
        phone1 = window.manager.phones[0]
        phone2 = window.manager.phones[1]
        
        print(f"\n3. Results:")
        print(f"   Handshakes completed: {results['handshakes']}")
        print(f"   Voice sessions started: {results['voice_started']}")
        print(f"   Decryption errors: {results['decrypt_errors']}")
        print(f"   Voice frames received: {results['frames_received']}")
        print(f"   Phone 1 received: {phone1['client'].voice_frame_counter} frames")
        print(f"   Phone 2 received: {phone2['client'].voice_frame_counter} frames")
        
        if (results['handshakes'] >= 2 and 
            results['voice_started'] >= 2 and 
            results['decrypt_errors'] == 0 and
            phone1['client'].voice_frame_counter > 0 and
            phone2['client'].voice_frame_counter > 0):
            print(f"\n✅ SUCCESS! Protocol working with proper framing!")
            print(f"   - Noise XK encryption ✓")
            print(f"   - Codec2 voice codec ✓")
            print(f"   - 4FSK modulation ✓")
            print(f"   - No fragmentation ✓")
        else:
            print(f"\n❌ Protocol test failed")
            if results['decrypt_errors'] > 0:
                print(f"   - Still getting decryption errors")
        
        app.quit()
    
    QTimer.singleShot(1000, test_sequence)
    sys.exit(app.exec_())

if __name__ == "__main__":
    main()